/*  L_MEM.H
 *  header file for use with the Dolphin Far Memory Manager Library
 *  last modification: 09-FEB-91
 *  version number at last modification: 1.20
 */

#ifndef DOLPHIN_MEMLIB_HEADER_INCLUDED

typedef   unsigned char        Uchar;
typedef   unsigned int         Uint;
typedef   unsigned long        Ulong;
typedef   unsigned char *      Str_ptr;
typedef   unsigned char far *  Far_str_ptr;
typedef   unsigned char huge * Huge_str_ptr;


#ifdef _FILE_DEFINED
/*  MSC STDIO.H #included  */
#define _STDIO_INCLUDED_
#endif

#ifdef __STDIO_DEF_
/*  TC STDIO.H #included  */
#define _STDIO_INCLUDED_
#endif

#ifndef _STDIO_INCLUDED_
#include <STDIO.H>          /*  for definition of FILE  */
#endif

/*  #defined constants  */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/*  error codes  */

#define NO_ERROR                      0
#define INSUFFICIENT_MEMORY          -1
#define NULL_POINTER                 -2
#define INVALID_BLOCK_ADDRESS        -3
#define CONTROL_BLOCK_DESTROYED      -4
#define CONTROL_BLOCK_CORRUPTED      -5
#define EXCESSIVE_REQUEST            -6
#define BLOCK_ALREADY_FREED          -7
#define INVALID_DECREASE             -8
#define ZERO_ARRAY_DIMENSION         -9
#define ZERO_ELEMENT_SIZE           -10
#define INVALID_ELEMENT_SIZE        -11
#define NULL_INIT_POINTER           -12
#define OUTPUT_ERROR                -13
#define LOG_ENTRY                   -14
#define LOG_CORRUPTED               -15
#define INCOMPLETE                  -16

/*  memory allocation strategies  */

#define FIRST_FIT 0
#define BEST_FIT  1
#define LAST_FIT  2

/*  for use with set_crit_err_termination_output()  */

#define NO_OUTPUT       0
#define SCREEN_OUTPUT   1
#define PRINTER_OUTPUT  2

#define REPORTS_1_AND_2  0
#define REPORT_1_ONLY    1
#define REPORT_2_ONLY    2
#define REPORT_3_ONLY    3

#define DOLPHIN_MEMLIB_HEADER_INCLUDED
#endif


/*  function declarations  */

int get_mem_alloc_strategy(void);
int set_mem_alloc_strategy(Uint n);

Far_str_ptr mem_alloc(Ulong n, int *err_flag);
Far_str_ptr mem_alloc_ch(Ulong n, Uchar ch, int *err_flag);
void mem_init(Far_str_ptr ptr, Uint size, char *init_val, int *err_flag);
void mem_free(void far *ptr, int *err_flag);
Far_str_ptr mem_resize(Far_str_ptr ptr, Ulong n, int *err_flag);
Far_str_ptr mem_resize_ch(Far_str_ptr ptr, Ulong n,
    Uchar ch, int *err_flag);
Far_str_ptr mem_increase(Far_str_ptr ptr, Uint increase, int *err_flag);
Far_str_ptr mem_increase_ch(Far_str_ptr ptr, Uint increase,
    Uchar ch, int *err_flag);
Far_str_ptr mem_decrease(Far_str_ptr ptr, Uint decrease, int *err_flag);

Ulong mem_sizeof(Far_str_ptr ptr, int *err_flag);

Ulong mem_max(int *err_flag);

Uint mem_log_entries(void);
void mem_log_on(void);
void mem_log_off(void);
int mem_log_is_on(void);
int mem_erase_log(int *err_flag);
void compact_log(int *err_flag);

Uint mem_log_report_1(FILE *output, Uint line_spacing,
    int formfeed, int *err_flag);
Uint mem_log_report_2(FILE *output, Uint line_spacing,
    int formfeed, int *err_flag);
void mem_log_report_3(FILE *output, Uint line_spacing,
    int formfeed, int *err_flag);
void mem_log_report_last(FILE *output, Uint line_spacing, int *err_flag);

Str_ptr mem_err_msg(int error, Str_ptr str);
void mem_log_msg(Str_ptr msg, int *err_flag);

Uint mem_free_all(int protected_block_id, int *err_flag);
int get_current_block_id(void);

void crit_err_termination_on(void);
void crit_err_termination_off(void);
int crit_err_termination_is_on(void);
void set_crit_err_termination_output(Uint output, Uint reports);
Uint get_crit_err_termination_output(void);

Far_str_ptr memcpy_f(void far *dest, void far *src, Ulong count);
int far *memcpy_int_f(int far *dest, int far *src, Ulong count);
long far *memcpy_long_f(long far *dest, long far *src, Ulong count);

Far_str_ptr mem_set(Far_str_ptr ptr, Uchar ch, int *err_flag);

Far_str_ptr memset_f(void far *ptr, char ch, Ulong count);
int far *memset_int_f(void far *ptr, int i, Ulong count);
long far *memset_long_f(void far *ptr, long i, Ulong count);

int get_mem_lib_version_number(void);

void huge *array1_alloc(Ulong n, Uint size, char *init_val, int *err_flag) ;
void huge * far *array2_alloc(Uint nrows, Uint ncols,
    Uint size, char *init_val, int *err_flag) ;
void huge * far * far *array3_alloc
    (Uint n1, Uint n2, Uint n3, Uint size, char *init_val, int *err_flag) ;
void huge * far * far * far *array4_alloc (Uint n1, Uint n2, Uint n3, Uint n4,
    Uint size, char *init_val, int *err_flag) ;
void huge * far * far * far * far *array5_alloc(Uint n1, Uint n2, Uint n3,
    Uint n4, Uint n5, Uint size, char *init_val, int *err_flag) ;

void array1_free(void huge * a, int *err_flag);
void array2_free(void huge * far * a, int *err_flag);
void array3_free(void huge * far * far * a, int *err_flag);
void array4_free(void huge * far * far * far * a, int *err_flag);
void array5_free(void huge * far * far * far * far * a, int *err_flag);

void array_expansion_on(void);
void array_expansion_off(void);
int array_expansion_is_on(void);

void hexdump(Str_ptr remark, Far_str_ptr start,
    Ulong num_bytes, FILE *output, int *err_flag);

void far_heap_dump(Far_str_ptr ptr, int n, Ulong bytes,
    FILE *output, int formfeed, int *err_flag);
